@@ -109,7 +109,12 @@ namespace eval diagnose {
             set config_options(profile_path) "${macports::user_home}/.bash_profile"
         }
         if {![info exists config_options(shell_location)]} {
-            set config_options(shell_location) /bin/bash
+            if {[info exists macports::sudo_user]} {
+                set username ${macports::sudo_user}
+            } else {
+                set username [exec id -un]
+            }
+            set config_options(shell_location) [lindex [exec /usr/bin/dscl . -read "/Users/${username}" shell] end]
         }
 
         # Start the checks
@@ -210,7 +215,7 @@ namespace eval diagnose {
         #           None
 
         if {!${diagnose::quiet}} {
-            ui_msg -nonewline "Checking for $string... "
+            ui_info -nonewline "Checking for $string... "
         }
     }
 
@@ -227,11 +232,11 @@ namespace eval diagnose {
 
             if {$result == 1} {
 
-                ui_msg "\[SUCCESS\]"
+                ui_info "\[SUCCESS\]"
                 return
             }
 
-            ui_msg "\[FAILED\]"
+            ui_info "\[FAILED\]"
         }
     }
 
@@ -396,27 +401,20 @@ namespace eval diagnose {
                         output "file '$file' on disk"
                     }
 
-                    if {![file exists $file]} {
+                    if {[catch {file type $file}]} {
                         if {$fancyOutput} {
                             $progress intermission
                         } else {
                             success_fail 0
                         }
-
-                        if {[catch {file lstat $file _}]} {
-                            ui_warn "couldn't find file '$file' for port '$name'. Please deactivate and reactivate the port to fix this issue."
-                        }
-                    } elseif {![file readable $file]} {
-                        if {$fancyOutput} {
-                            $progress intermission
-                        } else {
-                            success_fail 0
-                        }
-
-                        ui_warn "'$file' installed by port '$name' is currently not readable. Please try again. If this problem persists, please contact the mailing list."
+                        ui_warn "couldn't find file '$file' for port '$name'. Please deactivate and reactivate the port to fix this issue."
                     } elseif {!$fancyOutput} {
                         success_fail 1
                     }
+                    # TODO: check permissions against those in the port image.
+                    # Can't just check for readability because some files
+                    # (and/or their parent directories) should not be readable
+                    # by normal users for various reasons.
 
                     incr currentFile
                 }
@@ -438,17 +436,11 @@ namespace eval diagnose {
         # Returns:
         #           None
 
-        set apps [registry::entry imaged]
+        set ports [registry::entry imaged]
 
-        foreach app $apps {
-            set name [$app name]
-            output "'${name} @[$app version]_[$app revision][$app variants]'s tarball on disk"
-
-            if {![file exists [$app location]]} {
-                ui_warn "couldn't find the archive for '$name'. Please uninstall and reinstall this port."
-                success_fail 0
-            } else {
-                success_fail 1
+        foreach port $ports {
+            if {![file exists [$port location]]} {
+                ui_warn "couldn't find the archive for '[$port name] @[$port version]_[$port revision][$port variants]'. Please uninstall and reinstall this port."
             }
         }
     }
@@ -499,11 +491,8 @@ namespace eval diagnose {
         # Returns:
         #           None
 
-        output "X11.app on OS X 10.6 systems"
-
-        set mac_version ${macports::macosx_version}
-
-        if {$mac_version eq "10.6"} {
+        if {${macports::macosx_version} eq "10.6"} {
+            output "X11.app on OS X 10.6 systems"
 
             if {[file exists /Applications/X11.app]} {
                 ui_error "it seems you have Mac OS X 10.6 installed, and are using X11 from \"X11.app\". This has been known to cause issues. \
@@ -513,9 +502,8 @@ namespace eval diagnose {
                 success_fail 0
                 return
             }
+            success_fail 1
         }
-
-        success_fail 1
     }
 
     proc check_free_space {} {
@@ -696,6 +684,12 @@ namespace eval diagnose {
         # Returns:
         #           None.
 
+        set known_shells [list bash csh ksh sh tcsh zsh]
+        set shell_name [file tail $shell_loc]
+        if {$shell_name ni $known_shells} {
+            return
+        }
+
         set path ${macports::user_path}
         set split [split $path :]
 
@@ -708,34 +702,33 @@ namespace eval diagnose {
             return
 
         } else {
-            ui_warn "your \$PATH environment variable does not currently include, $port_loc/bin, which is where port is located. \
-                     Would you like to add $port_loc/bin to your \$PATH variable now? \[Y/N\]"
-            set input [gets stdin]
+            ui_warn "your \$PATH environment variable does not currently include $port_loc/bin, which is where port is located."
 
-            if {$input eq "y" || $input eq "Y"} {
-                # XXX: this should use the same paths and comments as the
-                # postflight script of the pkg installer. Maybe they could even
-                # share code?
-                ui_msg "Attempting to add $port_loc/bin to $profile_path"
+            # XXX Only works for bash. Should set default profile_path based on the shell.
+            if {[info exists macports::ui_options(questions_yesno)] && $shell_name eq "bash"} {
+                set retval [$macports::ui_options(questions_yesno) "" "DiagnoseFixPATH" "" n 0 "Would you like to add $port_loc/bin to your \$PATH variable now?"]
+                if {$retval} {
+                    # XXX: this should use the same paths and comments as the
+                    # postflight script of the pkg installer. Maybe they could even
+                    # share code?
+                    ui_msg "Attempting to add $port_loc/bin to $profile_path"
 
-                if {[file exists $profile_path]} {
-                    if {[file writable $profile_path]} {
-                        set fd [open $profile_path a]
-                        puts $fd "export PATH=$port_loc/bin:$port_loc/sbin:\$PATH"
-                        close $fd
+                    if {[file exists $profile_path]} {
+                        if {[file writable $profile_path]} {
+                            set fd [open $profile_path a]
+                            puts $fd "export PATH=$port_loc/bin:$port_loc/sbin:\$PATH"
+                            close $fd
 
-                        ui_msg "Added PATH properly. Please open a new terminal window to load the modified ${profile_path}."
+                            ui_msg "Added PATH properly. Please open a new terminal window to load the modified ${profile_path}."
+                        } else {
+                            ui_error "Can't write to ${profile_path}."
+                        }
                     } else {
-                        ui_error "Can't write to ${profile_path}."
+                        ui_error "$profile_path does not exist."
                     }
                 } else {
-                    ui_error "$profile_path does not exist."
+                    ui_msg "Not fixing your \$PATH variable."
                 }
-            } elseif {$input eq "n" || $input eq "N"} {
-                ui_msg "Not fixing your \$PATH variable."
-
-            } else {
-                ui_msg "Not a valid choice: $input"
             }
        }
    }
