@@ -122,6 +122,12 @@ namespace eval reclaim {
                     walk_files $currentPath $files_in_use unused
                 }
                 file {
+                    if {$item eq ".turd_MacPorts"} {
+                        # .turd_MacPorts files are created by MacPorts when creating the MacPorts
+                        # installer packages from the MacPorts port so that empty directories are
+                        # not deleted after destroot. Treat those files as if they were not there.
+                        continue
+                    }
                     if {[lsearch -exact -sorted $files_in_use $currentPath] == -1} {
                         ui_info "Found unused distfile $currentPath"
                         lappend unused $currentPath
@@ -349,7 +355,7 @@ namespace eval reclaim {
         set time [read_last_run_file]
 
         if {![string is integer -strict $time]} {
-            return
+            return 0
         }
 
         ui_debug "Checking time since last reclaim run"
@@ -360,7 +366,7 @@ namespace eval reclaim {
                 set retval [$macports::ui_options(questions_yesno) $msg "ReclaimPrompt" "" {y} 0 "Would you like to run it now?"]
                 if {$retval == 0} {
                     # User said yes, run port reclaim
-                    macports::reclaim_main
+                    return [macports::reclaim_main {}]
                 } else {
                     # User said no, ask again in two weeks
                     # Change this time frame if a consensus is agreed upon
@@ -370,6 +376,7 @@ namespace eval reclaim {
                 ui_warn $msg
             }
         }
+        return 0
     }
 
     proc sort_portlist_by_dependendents {portlist} {
@@ -447,9 +454,8 @@ namespace eval reclaim {
 
         } else {
 
-            ui_msg "Found inactive ports: [join $inactive_names {, }]."
             if {[info exists macports::ui_options(questions_multichoice)]} {
-                set retstring [$macports::ui_options(questions_multichoice) "Would you like to uninstall these ports?" "" $inactive_names]
+                set retstring [$macports::ui_options(questions_multichoice) "Would you like to uninstall these inactive ports?" "" $inactive_names]
 
                 if {[llength $retstring] > 0} {
                     foreach i $retstring {
