@@ -1,5 +1,4 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
-# $Id: portarchivefetch.tcl 140689 2015-09-30 06:27:47Z jmr@macports.org $
 #
 # Copyright (c) 2002 - 2003 Apple Inc.
 # Copyright (c) 2004 - 2013 The MacPorts Project
@@ -94,9 +93,9 @@ proc portarchivefetch::filter_sites {} {
             continue
         }
         if {$portfetch::mirror_sites::sites($site) ne {} &&
-            $portfetch::mirror_sites::archive_prefix($site) == $prefix &&
-            $portfetch::mirror_sites::archive_frameworks_dir($site) == $frameworks_dir &&
-            $portfetch::mirror_sites::archive_applications_dir($site) == $applications_dir &&
+            $portfetch::mirror_sites::archive_prefix($site) eq $prefix &&
+            $portfetch::mirror_sites::archive_frameworks_dir($site) eq $frameworks_dir &&
+            $portfetch::mirror_sites::archive_applications_dir($site) eq $applications_dir &&
             ![catch {archiveTypeIsSupported $portfetch::mirror_sites::archive_type($site)}]} {
             # using the archive type as a tag
             lappend ret ${site}::$portfetch::mirror_sites::archive_type($site)
@@ -137,7 +136,9 @@ proc portarchivefetch::checkarchivefiles {urls} {
 # returns full path to mirror list file
 proc portarchivefetch::get_full_archive_sites_path {} {
     global archive_sites.listfile archive_sites.listpath porturl
-    return [getportresourcepath $porturl [file join ${archive_sites.listpath} ${archive_sites.listfile}]]
+    # look up archive sites only from this ports tree,
+    # do not fallback to the default
+    return [getportresourcepath $porturl [file join ${archive_sites.listpath} ${archive_sites.listfile}] no]
 }
 
 # Perform the full checksites/checkarchivefiles sequence.
@@ -171,7 +172,7 @@ proc portarchivefetch::fetchfiles {args} {
     }
     set incoming_path [file join [option portdbpath] incoming]
     chownAsRoot $incoming_path
-    if {[info exists elevated] && $elevated == yes} {
+    if {[info exists elevated] && $elevated eq "yes"} {
         dropPrivileges
     }
 
@@ -180,10 +181,10 @@ proc portarchivefetch::fetchfiles {args} {
         lappend fetch_options -u
         lappend fetch_options "${archivefetch.user}:${archivefetch.password}"
     }
-    if {${archivefetch.use_epsv} != "yes"} {
+    if {${archivefetch.use_epsv} ne "yes"} {
         lappend fetch_options "--disable-epsv"
     }
-    if {${archivefetch.ignore_sslcert} != "no"} {
+    if {${archivefetch.ignore_sslcert} ne "no"} {
         lappend fetch_options "--ignore-ssl-cert"
     }
     if {$portverbose eq "yes"} {
@@ -207,7 +208,7 @@ proc portarchivefetch::fetchfiles {args} {
                 return -code error [format [msgcat::mc "%s must be writable"] $incoming_path]
             }
             if {!$sorted} {
-                portfetch::sortsites archivefetch_urls {} archive_sites
+                portfetch::sortsites archivefetch_urls archive_sites
                 set sorted yes
             }
             if {![info exists urlmap($url_var)]} {
@@ -216,6 +217,7 @@ proc portarchivefetch::fetchfiles {args} {
             }
             set failed_sites 0
             unset -nocomplain fetched
+            set lastError ""
             foreach site $urlmap($url_var) {
                 if {[string index $site end] ne "/"} {
                     append site "/[option archive.subdir]"
@@ -225,12 +227,21 @@ proc portarchivefetch::fetchfiles {args} {
                 ui_msg "$UI_PREFIX [format [msgcat::mc "Attempting to fetch %s from %s"] $archive ${site}]"
                 set file_url [portfetch::assemble_url $site $archive]
                 set effectiveURL ""
-                if {![catch {eval curl fetch --effective-url effectiveURL $fetch_options {$file_url} {"${incoming_path}/${archive}.TMP"}} result]} {
-                    # Successful fetch
+                try {
+                    curl fetch --effective-url effectiveURL {*}$fetch_options $file_url "${incoming_path}/${archive}.TMP"
                     set fetched 1
                     break
-                } else {
-                    ui_debug "[msgcat::mc "Fetching archive failed:"]: $result"
+                } catch {{POSIX SIG SIGINT} eCode eMessage} {
+                    ui_debug [msgcat::mc "Aborted fetching archive due to SIGINT"]
+                    file delete -force "${incoming_path}/${archive}.TMP"
+                    throw
+                } catch {{POSIX SIG SIGTERM} eCode eMessage} {
+                    ui_debug [msgcat::mc "Aborted fetching archive due to SIGTERM"]
+                    file delete -force "${incoming_path}/${archive}.TMP"
+                    throw
+                } catch {{*} eCode eMessage} {
+                    ui_debug [msgcat::mc "Fetching archive failed: %s" $eMessage]
+                    set lastError $eMessage
                     file delete -force "${incoming_path}/${archive}.TMP"
                     incr failed_sites
                     if {$failed_sites > 2 && ![tbool ports_binary_only] && ![_archive_available]} {
@@ -243,7 +254,7 @@ proc portarchivefetch::fetchfiles {args} {
                 set signature "${incoming_path}/${archive}.rmd160"
                 ui_msg "$UI_PREFIX [format [msgcat::mc "Attempting to fetch %s from %s"] ${archive}.rmd160 $site]"
                 # reusing $file_url from the last iteration of the loop above
-                if {[catch {eval curl fetch --effective-url effectiveURL $fetch_options {${file_url}.rmd160} {$signature}} result]} {
+                if {[catch {curl fetch --effective-url effectiveURL {*}$fetch_options ${file_url}.rmd160 $signature} result]} {
                     ui_debug "$::errorInfo"
                     return -code error "Failed to fetch signature for archive: $result"
                 }
@@ -285,7 +296,11 @@ proc portarchivefetch::fetchfiles {args} {
         return 0
     }
     if {([info exists ports_binary_only] && $ports_binary_only eq "yes") || [_archive_available]} {
-        return -code error "archivefetch failed for [option subport] @[option version]_[option revision][option portvariants]"
+        if {[info exists lastError] && $lastError ne ""} {
+            error [msgcat::mc "version @[option version]_[option revision][option portvariants]: %s" $lastError]
+        } else {
+            error "version @[option version]_[option revision][option portvariants]"
+        }
     } else {
         return 0
     }
@@ -306,6 +321,8 @@ proc portarchivefetch::archivefetch_start {args} {
     }
     if {[info exists all_archive_files] && [llength $all_archive_files] > 0} {
         ui_msg "$UI_PREFIX [format [msgcat::mc "Fetching archive for %s"] $subport]"
+    } elseif {[tbool ports_binary_only]} {
+        error "Binary-only mode requested with no usable archive sites configured"
     }
     portfetch::check_dns
 }
