@@ -1,6 +1,5 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
 # portdestroot.tcl
-# $Id$
 #
 # Copyright (c) 2002 - 2003 Apple Inc.
 # Copyright (c) 2004 - 2005 Robert Shaw <rshaw@opendarwin.org>
@@ -131,12 +130,12 @@ proc portdestroot::destroot_start {args} {
     set oldmask [umask ${destroot.umask}]
     set mtree [findBinary mtree ${portutil::autoconf::mtree_path}]
 
-    if { ${destroot.clean} == "yes" } {
+    if { ${destroot.clean} eq "yes" } {
         delete "${destroot}"
     }
 
     file mkdir "${destroot}"
-    if { ${os.platform} == "darwin" } {
+    if { ${os.platform} eq "darwin" } {
         system "cd \"${destroot}\" && ${mtree} -e -U -f [file join ${portsharepath} install macosx.mtree]"
         file mkdir "${destroot}${applications_dir}"
         file mkdir "${destroot}${frameworks_dir}"
@@ -195,7 +194,7 @@ proc portdestroot::destroot_finish {args} {
         if {${destroot.delete_la_files}} {
             ui_debug "Removing [file tail $fullpath]"
             file delete -force ${fullpath}
-        } else {
+        } elseif {[file type $fullpath] eq "file"} {
             ui_debug "Clearing dependency_libs in [file tail $fullpath]"
             reinplace "/dependency_libs/ s/'.*'/''/" ${fullpath}
         }
@@ -247,16 +246,16 @@ proc portdestroot::destroot_finish {args} {
                             set found 1
                             system "cd ${manpath} && \
                             $gunzip -f [file join ${mandir} ${gzfile}] && \
-                            $gzip -9vf [file join ${mandir} ${manfile}]"
+                            $gzip -9vnf [file join ${mandir} ${manfile}]"
                         } elseif {[regexp "^(.*\[.\]${manindex}\[a-z\]*)\[.\]bz2\$" ${manfile} bz2file manfile]} {
                             set found 1
                             system "cd ${manpath} && \
                             $bunzip2 -f [file join ${mandir} ${bz2file}] && \
-                            $gzip -9vf [file join ${mandir} ${manfile}]"
+                            $gzip -9vnf [file join ${mandir} ${manfile}]"
                         } elseif {[regexp "\[.\]${manindex}\[a-z\]*\$" ${manfile}]} {
                             set found 1
                             system "cd ${manpath} && \
-                            $gzip -9vf [file join ${mandir} ${manfile}]"
+                            $gzip -9vnf [file join ${mandir} ${manfile}]"
                         }
                         set gzmanfile ${manfile}.gz
                         set gzmanfilepath [file join ${mandirpath} ${gzmanfile}]
@@ -316,7 +315,7 @@ proc portdestroot::destroot_finish {args} {
     }
 
     # test for violations of mtree
-    if { ${destroot.violate_mtree} != "yes" } {
+    if { ${destroot.violate_mtree} ne "yes" } {
         ui_debug "checking for mtree violations"
         set mtree_violation "no"
 
@@ -341,7 +340,7 @@ proc portdestroot::destroot_finish {args} {
                         if {$pfile eq "." || $pfile eq ".."} {
                             continue
                         }
-                        if {[lsearch -exact $prefixPaths $pfile] == -1} {
+                        if {$pfile ni $prefixPaths} {
                             ui_warn "violation by [file join $dfile $pfile]"
                             set mtree_violation "yes"
                         }
@@ -379,7 +378,7 @@ proc portdestroot::destroot_finish {args} {
         }
 
         # abort here only so all violations can be observed
-        if { ${mtree_violation} != "no" } {
+        if { ${mtree_violation} ne "no" } {
             ui_warn "[format [msgcat::mc "%s violates the layout of the ports-filesystems!"] [option subport]]"
             ui_warn "Please fix or indicate this misbehavior (if it is intended), it will be an error in future releases!"
             # error "mtree violation!"
