@@ -1,8 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
-# portfetch.tcl
-# $Id$
 #
-# Copyright (c) 2004 - 2012 The MacPorts Project
+# Copyright (c) 2004 - 2014, 2016 The MacPorts Project
 # Copyright (c) 2002 - 2003 Apple Inc.
 # All rights reserved.
 #
@@ -70,7 +68,7 @@ default fetch.type standard
 default bzr.cmd {[findBinary bzr $portutil::autoconf::bzr_path]}
 default bzr.dir {${workpath}}
 default bzr.revision {-1}
-default bzr.pre_args {"--builtin --no-aliases checkout --lightweight"}
+default bzr.pre_args {"--builtin --no-aliases checkout --lightweight --verbose"}
 default bzr.args ""
 default bzr.post_args {"-r ${bzr.revision} ${bzr.url} ${worksrcdir}"}
 
@@ -138,6 +136,9 @@ proc portfetch::set_extract_type {option action args} {
         switch $option {
             use_bzip2 {
                 set extract.suffix .tar.bz2
+                if {![catch {findBinary lbzip2} result]} {
+                    depends_extract-append bin:lbzip2:lbzip2
+                }
             }
             use_lzma {
                 set extract.suffix .tar.lzma
@@ -180,7 +181,7 @@ proc portfetch::set_fetch_type {option action args} {
                 depends_fetch-append bin:cvs:cvs
             }
             svn {
-                if {${os.platform} == "darwin" && ${os.major} >= 10} {
+                if {${os.major} >= 10 || ${os.platform} ne "darwin"} {
                     depends_fetch-append bin:svn:subversion
                 } else {
                     depends_fetch-append port:subversion
@@ -199,7 +200,7 @@ proc portfetch::set_fetch_type {option action args} {
 proc portfetch::find_svn_path {args} {
     global prefix os.platform os.major
     # Snow Leopard is the first Mac OS X version to include a recent enough svn (1.6.x) to support the --trust-server-cert option.
-    if {${os.platform} == "darwin" && ${os.major} >= 10} {
+    if {${os.major} >= 10 || ${os.platform} ne "darwin"} {
         return [findBinary svn $portutil::autoconf::svn_path]
     } else {
         return ${prefix}/bin/svn
@@ -309,10 +310,10 @@ proc portfetch::bzrfetch {args} {
             return -code error [msgcat::mc "Bazaar checkout failed"]
         }
     } finally {
-        if ([info exists orig_http_proxy]) {
+        if {[info exists orig_http_proxy]} {
             set env(http_proxy) ${orig_http_proxy}
         }
-        if ([info exists orig_https_proxy]) {
+        if {[info exists orig_https_proxy]} {
             set env(HTTPS_PROXY) ${orig_https_proxy}
         }
     }
@@ -332,7 +333,7 @@ proc portfetch::cvsfetch {args} {
            patch_sites patchfiles filespath
 
     set cvs.args "${cvs.method} ${cvs.args}"
-    if {${cvs.method} == "export" && ![string length ${cvs.tag}] && ![string length ${cvs.date}]} {
+    if {${cvs.method} eq "export" && ![string length ${cvs.tag}] && ![string length ${cvs.date}]} {
         set cvs.tag "HEAD"
     }
     if {[string length ${cvs.tag}]} {
@@ -427,10 +428,10 @@ proc portfetch::gitfetch {args} {
     global worksrcpath patchfiles \
            git.url git.branch git.sha1 git.cmd
 
-    set options "-q"
-    if {[string length ${git.branch}] == 0} {
+    set options "--progress"
+    if {${git.branch} eq ""} {
         # if we're just using HEAD, we can make a shallow repo
-        set options "$options --depth=1"
+        append options " --depth=1"
     }
     set cmdstring "${git.cmd} clone $options ${git.url} ${worksrcpath} 2>&1"
     ui_debug "Executing: $cmdstring"
@@ -438,7 +439,7 @@ proc portfetch::gitfetch {args} {
         return -code error [msgcat::mc "Git clone failed"]
     }
 
-    if {[string length ${git.branch}] > 0} {
+    if {${git.branch} ne ""} {
         set env "GIT_DIR=${worksrcpath}/.git GIT_WORK_TREE=${worksrcpath}"
         set cmdstring "$env ${git.cmd} checkout -q ${git.branch} 2>&1"
         ui_debug "Executing $cmdstring"
@@ -482,7 +483,7 @@ proc portfetch::hgfetch {args} {
 proc portfetch::fetchfiles {args} {
     global distpath all_dist_files UI_PREFIX \
            fetch.user fetch.password fetch.use_epsv fetch.ignore_sslcert fetch.remote_time \
-           portverbose usealtworkpath altprefix
+           portverbose
     variable fetch_urls
     variable urlmap
 
@@ -491,13 +492,13 @@ proc portfetch::fetchfiles {args} {
         lappend fetch_options -u
         lappend fetch_options "${fetch.user}:${fetch.password}"
     }
-    if {${fetch.use_epsv} != "yes"} {
+    if {${fetch.use_epsv} ne "yes"} {
         lappend fetch_options "--disable-epsv"
     }
-    if {${fetch.ignore_sslcert} != "no"} {
+    if {${fetch.ignore_sslcert} ne "no"} {
         lappend fetch_options "--ignore-ssl-cert"
     }
-    if {${fetch.remote_time} != "no"} {
+    if {${fetch.remote_time} ne "no"} {
         lappend fetch_options "--remote-time"
     }
     if {$portverbose eq "yes"} {
@@ -515,14 +516,6 @@ proc portfetch::fetchfiles {args} {
             if {![file writable $distpath]} {
                 return -code error [format [msgcat::mc "%s must be writable"] $distpath]
             }
-            if {!$usealtworkpath && [file isfile ${altprefix}${distpath}/${distfile}]} {
-                if {[catch {file link -hard "${distpath}/${distfile}" "${altprefix}${distpath}/${distfile}"}]} {
-                    ui_debug "failed to hardlink ${distfile} into distpath, copying instead"
-                    file copy "${altprefix}${distpath}/${distfile}" "${distpath}/${distfile}"
-                }
-                ui_info "Found $distfile in ${altprefix}${distpath}"
-                continue
-            }
             if {!$sorted} {
                 sortsites fetch_urls master_sites
                 set sorted yes
@@ -532,20 +525,28 @@ proc portfetch::fetchfiles {args} {
                 set urlmap($url_var) $urlmap(master_sites)
             }
             unset -nocomplain fetched
+            set lastError ""
             foreach site $urlmap($url_var) {
                 ui_notice "$UI_PREFIX [format [msgcat::mc "Attempting to fetch %s from %s"] $distfile $site]"
                 set file_url [portfetch::assemble_url $site $distfile]
-                if {![catch {eval curl fetch $fetch_options {$file_url} {"${distpath}/${distfile}.TMP"}} result] &&
-                    ![catch {file rename -force "${distpath}/${distfile}.TMP" "${distpath}/${distfile}"} result]} {
+                try -pass_signal {
+                    curl fetch {*}$fetch_options $file_url "${distpath}/${distfile}.TMP"
+                    file rename -force "${distpath}/${distfile}.TMP" "${distpath}/${distfile}"
                     set fetched 1
                     break
-                } else {
-                    ui_debug "[msgcat::mc "Fetching distfile failed"]: $result"
+                } catch {{*} eCode eMessage} {
+                    ui_debug [msgcat::mc "Fetching distfile failed: %s" $eMessage]
+                    set lastError $eMessage
+                } finally {
                     file delete -force "${distpath}/${distfile}.TMP"
                 }
             }
             if {![info exists fetched]} {
-                return -code error [msgcat::mc "fetch failed"]
+                if {$lastError ne ""} {
+                    error $lastError
+                } else {
+                    error [msgcat::mc "fetch failed"]
+                }
             }
         }
     }
@@ -618,7 +619,7 @@ proc portfetch::fetch_main {args} {
     global all_dist_files fetch.type
 
     # Check for files, download if necessary
-    if {![info exists all_dist_files] && "${fetch.type}" == "standard"} {
+    if {![info exists all_dist_files] && "${fetch.type}" eq "standard"} {
         return 0
     }
 
